"use strict";

var invariant = require("invariant");

var writtenNumber = require("written-number/lib/standalone");

function writtenNumberCurrency() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref = config || {},
      lang = _ref.lang,
      baseUnitAmountArg = _ref.baseUnitAmount,
      smallUnitAmountArg = _ref.smallUnitAmount,
      baseUnitFormatter = _ref.baseUnitFormatter,
      _ref$linker = _ref.linker,
      linker = _ref$linker === void 0 ? "" : _ref$linker,
      smallUnitFormatter = _ref.smallUnitFormatter;

  invariant(lang, "config.lang is required");
  writtenNumber.defaults.lang = lang;
  return function () {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        baseUnitAmountArg = _ref2.baseUnitAmount,
        smallUnitAmountArg = _ref2.smallUnitAmount;

    var baseUnitAmount, smallUnitAmount;

    if (baseUnitAmountArg == null && smallUnitAmountArg != null) {
      smallUnitAmount = prepareNumber(smallUnitAmountArg, "smallUnitAmount");
      validateFunction(smallUnitFormatter, "smallUnitFormatter");
    } else if (baseUnitAmountArg != null && smallUnitAmountArg == null) {
      baseUnitAmount = prepareNumber(baseUnitAmountArg, "baseUnitAmount");
      validateFunction(baseUnitFormatter, "baseUnitFormatter");
    } else if (baseUnitAmountArg == null && smallUnitAmountArg == null) {
      invariant("Please provide a baseUnitAmount or a smallUnitAmount");
    } else {
      baseUnitAmount = prepareNumber(baseUnitAmountArg, "baseUnitAmount");
      smallUnitAmount = prepareNumber(smallUnitAmountArg, "smallUnitAmount");
      validateFunction(baseUnitFormatter, "baseUnitFormatter");
      validateFunction(smallUnitFormatter, "smallUnitFormatter");
    }

    if (baseUnitAmount == null) {
      return format(smallUnitAmount, smallUnitFormatter);
    }

    if (smallUnitAmount == null) {
      return format(baseUnitAmount, baseUnitFormatter);
    }

    return link(format(baseUnitAmount, baseUnitFormatter), format(smallUnitAmount, smallUnitFormatter), linker);
  };
}

function link(left, right, linker) {
  return linker ? "".concat(left, " ").concat(linker, " ").concat(right) : "".concat(left, " ").concat(right);
}

function format(amount, formatter) {
  return writtenNumber(amount) + " " + formatter(amount);
}

function validateFunction(n, name) {
  if (typeof n !== "function") {
    invariant(false, "config.".concat(name, " should be a valid function"));
  }
}

function prepareNumber(n, name) {
  n = Number(n);

  if (Number.isNaN(n)) {
    invariant(false, "config.".concat(name, " should be valid number"));
  }

  if (typeof n === "number" && n < 0) {
    invariant(false, "config.".concat(name, " should be greater than or equal to zero"));
  }

  return Math.round(n);
}

module.exports = writtenNumberCurrency;